@@ -1,19 +1,19 @@
-from typing import Any, Dict, Optional, TYPE_CHECKING
+from typing import Any, Dict, List, NewType, Optional, TYPE_CHECKING
+
 
 if TYPE_CHECKING:
     from dataclasses import dataclass
 else:
     from zuper_typing import dataclass
 
-
 import numpy as np
 
-from zuper_typing.literal import make_Literal  # XXX
 from .basics import InteractionProtocol
 from .protocol_agent import EpisodeStart
 
 __all__ = [
     "RobotState",
+    "GetDuckieState",
     "RobotName",
     "RobotObservations",
     "RobotInterfaceDescription",
@@ -36,9 +36,12 @@ __all__ = [
     "StateDump",
     "JPGImage",
     "DumpState",
-    "MOTION_PARKED",
-    "MOTION_MOVING",
+    "DuckieState",
     "SpawnDuckie",
+    "PROTOCOL_FULL",
+    "PROTOCOL_LEARNING",
+    "PROTOCOL_NORMAL",
+    "Termination",
 ]
 
 RobotName = str
@@ -79,12 +82,6 @@ class GetRobotObservations:
     t_effective: float
 
 
-@dataclass
-class GetRobotState:
-    robot_name: RobotName
-    t_effective: float
-
-
 @dataclass
 class RobotObservations:
     robot_name: RobotName
@@ -92,6 +89,12 @@ class RobotObservations:
     observations: Any
 
 
+@dataclass
+class GetRobotState:
+    robot_name: RobotName
+    t_effective: float
+
+
 @dataclass
 class RobotState:
     robot_name: RobotName
@@ -99,6 +102,26 @@ class RobotState:
     state: Any
 
 
+@dataclass
+class GetDuckieState:
+    duckie_name: str
+    t_effective: float
+
+
+@dataclass
+class DuckieState:
+    duckie_name: str
+    t_effective: float
+    state: Any
+
+
+@dataclass
+class Termination:
+    when: float
+    desc: str
+    code: str
+
+
 @dataclass
 class SimulationState:
     """
@@ -110,8 +133,9 @@ class SimulationState:
     """
 
     done: bool
-    done_why: Optional[str]
-    done_code: Optional[str]
+    terminations: Dict[str, Optional[Termination]]
+    # done_why: Optional[str]
+    # done_code: Optional[str]
 
 
 @dataclass
@@ -170,20 +194,33 @@ class StateDump:
     state: object
 
 
-MOTION_PARKED = "parked"
-MOTION_MOVING = "moving"
-NPMotion = make_Literal(MOTION_PARKED, MOTION_MOVING)
+#
+# MOTION_PARKED = "parked"
+# MOTION_MOVING = "moving"
+# NPMotion = make_Literal(MOTION_PARKED, MOTION_MOVING)
+
+ProtocolDesc = NewType("ProtocolDesc", str)
+PROTOCOL_NORMAL = ProtocolDesc("PROTOCOL_NORMAL")
+""" only the things that are visible """
+PROTOCOL_FULL = ProtocolDesc("PROTOCOL_FULL")
+""" Full state observation"""
+PROTOCOL_LEARNING = ProtocolDesc("PROTOCOL_LEARNING")
+""" Also includes a feedback signal"""
 
 
 @dataclass
 class ScenarioRobotSpec:
-    description: str
-    playable: bool
     configuration: RobotConfiguration
     color: str
 
-    # if not playable
-    motion: Optional[NPMotion]
+    description: str
+
+    # if playable
+    controllable: bool
+    protocol: Optional[ProtocolDesc]
+
+    # # if not playable
+    # motion: Optional[NPMotion]
 
 
 @dataclass
@@ -192,12 +229,25 @@ class ScenarioDuckieSpec:
     pose: np.ndarray  # SE2
 
 
+@dataclass
+class Scenario:
+    # Specification of the environments
+    scenario_name: str
+    environment: Any
+    player_robots: List[str]
+    robots: Dict[str, ScenarioRobotSpec]
+    duckies: Dict[str, ScenarioDuckieSpec]
+
+
 @dataclass
 class SpawnRobot:
     playable: bool
+    """ we will send command """
+    owned_by_player: bool
+    """ It is owned by player. Terminate when they all terminate. """
     robot_name: RobotName
     configuration: RobotConfiguration
-    motion: Optional[NPMotion]
+    # motion: Optional[NPMotion]
 
 
 @dataclass
@@ -207,15 +257,6 @@ class SpawnDuckie:
     pose: np.ndarray
 
 
-@dataclass
-class Scenario:
-    # Specification of the environments
-    scenario_name: str
-    environment: Any
-    robots: Dict[str, ScenarioRobotSpec]
-    duckies: Dict[str, ScenarioDuckieSpec]
-
-
 description = """\
 
 A "scenario maker" is an object that is able to create robot simulations scenarios.
@@ -314,6 +355,7 @@ protocol_simulator = InteractionProtocol(
                     (in:get_robot_observations ;  out:robot_observations) |
                     (in:get_robot_performance ;  out:robot_performance) |
                     (in:get_robot_state ;  out:robot_state) |
+                    (in:get_duckie_state ;  out:duckie_state) |
                     (in:get_sim_state ;  out:sim_state) |
                     (in:dump_state   ;  out:state_dump) |
                     (in:get_ui_image ; out:ui_image) 
@@ -336,6 +378,7 @@ protocol_simulator = InteractionProtocol(
         "set_robot_commands": SetRobotCommands,
         "get_robot_observations": GetRobotObservations,
         "get_robot_state": GetRobotState,
+        "get_duckie_state": GetDuckieState,
         "get_ui_image": type(None),
         # Dump state information
         "dump_state": DumpState,
@@ -343,6 +386,7 @@ protocol_simulator = InteractionProtocol(
     outputs={
         "robot_observations": RobotObservations,
         "robot_state": RobotState,
+        "duckie_state": DuckieState,
         "robot_performance": RobotPerformance,
         "robot_interface_description": RobotInterfaceDescription,
         "sim_state": SimulationState,
